/*
 * $Id: ResourcesBase.java 354330 2005-12-06 06:05:19Z niallp $
 * $Revision: 354330 $
 * $Date: 2005-12-06 01:05:19 -0500 (Tue, 06 Dec 2005) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.Locale;

import org.apache.commons.resources.Resources;
import org.apache.commons.resources.ResourcesException;
import org.apache.commons.resources.ResourcesKeyException; // for javadoc/checkstyle

/**
 * <p>Convenience base class for 
 * {@link org.apache.commons.resources.Resources} implementations.</p>
 *
 * <p>Default implementations of the content retrieval methods are provided
 * for all methods except <code>getObject()</code>.  The default methods for
 * the other content retrieval methods are coded in terms of
 * <code>getString()</code>, on the assumption that most uses of
 * {@link org.apache.commons.resources.Resources} are of this type.  
 * However, they can be easily overridden as needed.</p>
 *
 * @see org.apache.commons.resources.impl.CollectionResourcesBase
 * @see org.apache.commons.resources.impl.JDBCResources
 * @see org.apache.commons.resources.impl.PropertyResources
 * @see org.apache.commons.resources.impl.ResourceBundleResources
 * @see org.apache.commons.resources.impl.WebappPropertyResources
 * @see org.apache.commons.resources.impl.WebappXMLResources
 * @see org.apache.commons.resources.impl.XMLResources
 */
public abstract class ResourcesBase implements Resources {


    // ----------------------------------------------------------- Constructors


    /**
     * <p>Create a new {@link org.apache.commons.resources.Resources} instance 
     * with no name.</p>
     */
    public ResourcesBase() {

        this(null);

    }


    /**
     * <p>Create a new {@link org.apache.commons.resources.Resources} instance 
     * with the specified logical name.</p>
     *
     * @param name Logical name of the new instance
     */
    public ResourcesBase(String name) {

        this.name = name;

    }


    // ----------------------------------------------------- Instance Variables


    /**
     * <p>The logical name of this {@link org.apache.commons.resources.Resources} 
     * instance.</p>
     */
    private String name = null;


    /**
     * <p>Flag indicating whether resource getter methods should return
     * <code>null</code> (instead of throwing an exception) on invalid
     * key values.</p>
     */
    private boolean returnNull = true;

    /**
     * Buffer size for creating char and byte arrays.
     */
    private int bufferSize = 500;

    // ------------------------------------------------------ Lifecycle Methods


    /**
     * <p>This must be called to initialize the data content of this
     * {@link org.apache.commons.resources.Resources} instance, before 
     * any of the <code>getXxx()</code> methods are called.</p>
     *
     * <p>The default implementation does nothing.</p>
     *
     * @exception ResourcesException if an error occurs during initialization
     */
    public void init() {

         // The default implementation does nothing

    }


    /**
     * <p>This method must be called when the manager of this resource
     * decides that it's no longer needed.  After this method is called,
     * no further calls to any of the <code>getXxx()</code> methods are
     * allowed.</p>
     *
     * <p>The default implementation does nothing.</p>
     *
     * @exception ResourcesException if an error occurs during finalization
     */
    public void destroy() {

        // The default implementation does nothing

    }


    // ------------------------------------------------------------- Properties


    /**
     * <p>Return an <code>Iterator</code> over the defined keys in this
     * {@link org.apache.commons.resources.Resources} instance.</p>
     *
     * @return The keys contained in this resources instance.
     */
    public abstract Iterator getKeys();


    /**
     * <p>Return the logical name of this {@link org.apache.commons.resources.Resources} 
     * instance.</p>
     *
     * @return The name of this resources instance.
     */
    public String getName() {

        return (this.name);

    }


    /**
     * <p>Return <code>true</code> if resource getter methods will return
     * <code>null</code> instead of throwing an exception on invalid
     * key values.</p>
     * @return 'true' if null is returned for invalid key values.
     */
    public boolean isReturnNull() {

        return (this.returnNull);

    }


    /**
     * <p>Set a flag determining whether resource getter methods should
     * return <code>null</code> instead of throwing an exception on
     * invalid key values.</p>
     *
     * @param returnNull The new flag value
     */
    public void setReturnNull(boolean returnNull) {

        this.returnNull = returnNull;

    }


    /**
     * <p>Return the size of the buffer to use when converting
     * InputStream or Reader objects.</p>
     *
     * @return The buffer size.
     */
    public int getBufferSize() {

        return bufferSize;

    }

    /**
     * <p>Set the size of the buffer to use when converting
     * InputStream or Reader objects.</p>
     *
     * @param bufferSize The buffer size.
     */
    public void setBufferSize(int bufferSize) {

        this.bufferSize = bufferSize;

    }


    // ---------------------------------------------- Content Retrieval Methods


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * byte array, localized based on the specified <code>locale</code>.
     * </p>
     *
     * <p>The default implementation calls <code>getString()</code> and
     * converts the value to a byte array.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @return content for a specified key.
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public byte[] getBytes(String key, Locale locale) {

        Object value = getObject(key, locale);
        if (value == null) {
            return (null);
        } else if (value instanceof String) {
            return ((String)value).getBytes();
        } else if (value instanceof Reader) {
            char[] chars = getChars((Reader)value);
            if (chars == null) {
                return (byte[])checkThrow(key);
            }
            return new String(chars).getBytes();
        } else if (value instanceof InputStream) {
            byte[] bytes = getBytes((InputStream)value);
            if (bytes == null) {
                return (byte[])checkThrow(key);
            }
            return bytes;
        } else if (value instanceof byte[]) {
            return (byte[])value;
        } else {
            return value.toString().getBytes();
        }

    }


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * InputStream, localized based on the specified <code>locale</code>.
     * </p>
     *
     * <p>The default implementation calls <code>getsBytes()</code>
     * and returns an input stream over the resulting byte array.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @return content for a specified key.
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public InputStream getInputStream(String key, Locale locale) {

        Object value = getObject(key, locale);
        if (value == null) {
            return (null);
        } else if (value instanceof String) {
            byte[] bytes = ((String)value).getBytes();
            return new ByteArrayInputStream(bytes);
        } else if (value instanceof Reader) {
            char[] chars = getChars((Reader)value);
            if (chars == null) {
                return (InputStream)checkThrow(key);
            }
            byte[] bytes = (new String(chars)).getBytes();
            return new ByteArrayInputStream(bytes);
        } else if (value instanceof InputStream) {
            return (InputStream)value;
        } else if (value instanceof byte[]) {
            return new ByteArrayInputStream((byte[])value);
        } else {
            byte[] bytes = value.toString().getBytes();
            return new ByteArrayInputStream(bytes);
        }

    }


    /**
     * <p>Return the content for the specified <code>key</code> as an
     * Object, localized based on the specified <code>locale</code>.
     * </p>
     *
     * <p>There is no default implementation of this method.  Concrete
     * subclasses must provide such an implementation.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @return content for a specified key.
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public abstract Object getObject(String key, Locale locale);


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * Reader, localized based on the specified <code>locale</code>.
     * </p>
     *
     * <p>The default implementation calls <code>getString()</code>
     * and returns a reader over the resulting characters.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @return content for a specified key.
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public Reader getReader(String key, Locale locale) {

        Object value = getObject(key, locale);
        if (value == null) {
            return (null);
        } else if (value instanceof String) {
            return new StringReader((String)value);
        } else if (value instanceof Reader) {
            return (Reader)value;
        } else if (value instanceof InputStream) {
            return new InputStreamReader((InputStream)value);
        } else if (value instanceof byte[]) {
            InputStream bais = new ByteArrayInputStream((byte[])value);
            return new InputStreamReader(bais);
        } else {
            return new StringReader(value.toString());
        }

    }


    /**
     * <p>Return the content for the specified <code>key</code> as a
     * String, localized based on the specified <code>locale</code>.
     * </p>
     *
     * <p>The default implementation calls <code>getObject()</code>
     * and converts the result to a String if necessary.</p>
     *
     * @param key Identifier for the requested content
     * @param locale Locale with which to localize retrieval,
     *  or <code>null</code> for the default Locale
     * @return content for a specified key.
     *
     * @exception ResourcesException if an error occurs retrieving or
     *  returning the requested content
     * @exception ResourcesKeyException if the no value for the specified
     *  key was found, and <code>isReturnNull()</code> returns
     *  <code>false</code>
     */
    public String getString(String key, Locale locale) {

        Object value = getObject(key, locale);
        if (value == null) {
            return (null);
        } else if (value instanceof String) {
            return ((String) value);
        } else if (value instanceof Reader) {
            char[] chars = getChars((Reader)value);
            if (chars == null) {
                return (String)checkThrow(key);
            }
            return new String(chars);
        } else if (value instanceof InputStream) {
            byte[] bytes = getBytes((InputStream)value);
            if (bytes == null) {
                return (String)checkThrow(key);
            }
            return new String(bytes);
        } else if (value instanceof byte[]) {
            return new String((byte[])value);
        } else {
            return (value.toString());
        }

    }


    /**
     * Convert a Reader to a char array.
     */
    private char[] getChars(Reader reader) {

        char[] array = null;
        int arrayLth = 0;
        try {
            while (true) {
                char[] buffer = new char[bufferSize]; 
                int bufferLth = reader.read(buffer);
                if (bufferLth < 0) {
                    break;
                }
                if (array == null && bufferLth == buffer.length) {
                    array = buffer;
                } else {
                    char[] newArray = new char[arrayLth + bufferLth];
                    if (array != null) {
                        System.arraycopy(array, 0, newArray, 0, arrayLth);
                    }
                    System.arraycopy(buffer, 0, newArray, arrayLth, bufferLth);
                    array = newArray;
                }
                arrayLth = array.length;
            }
        } catch(IOException e) {
            throw new ResourcesException("Error reading Reader", e);
        }

        return array;

    }

    /**
     * Convert an InputStream to a byte array.
     */
    private byte[] getBytes(InputStream inputStream) {

        byte[] array = null;
        int arrayLth = 0;
        try {
            while (true) {
                byte[] buffer = new byte[bufferSize]; 
                int bufferLth = inputStream.read(buffer);
                if (bufferLth < 0) {
                    break;
                }
                if (array == null && bufferLth == buffer.length) {
                    array = buffer;
                } else {
                    byte[] newArray = new byte[arrayLth + bufferLth];
                    if (array != null) {
                        System.arraycopy(array, 0, newArray, 0, arrayLth);
                    }
                    System.arraycopy(buffer, 0, newArray, arrayLth, bufferLth);
                    array = newArray;
                }
                arrayLth = array.length;
            }
        } catch(IOException e) {
            throw new ResourcesException("Error reading InputStream", e);
        }
        return array;

    }

    /**
     * Check whether this Resources instance is
     * configured to return null or throw a ResourcesKeyException.
     * @param key Identifier for the requested content
     * @return 'null' if returnNull is 'true' otherwise throws
     *        a ResourcesKeyException.
     */
    private Object checkThrow(String key) {
        if (isReturnNull()) {
            return (null);
        } else {
            throw new ResourcesKeyException(key);
        }
    }

}
